#include "enigma/enigma.h"
#include <sstream>
#include <algorithm>

double index(const std::string &message)
{
    int total = 0;
    std::array<double, 26> counts;
    counts.fill(0.0);
    for (auto letter : message)
    {
        int letterCode = letter - 'A';
        if (letterCode < 0 && letterCode > 25)
            continue;

        counts[letterCode]++;
        total++;
    }

    // std::cout << "Frequencies :" << std::endl;

    double sum = 0;
    char index = 'A';
    for (double count : counts)
    {
        // std::cout << "Count of " << index << ":" << count << std::endl;
        sum += count * (count - 1);
        index++;
    }
    return sum / (total * (total - 1));
}

std::pair<double, std::array<int, 3>> try_all_keys(const std::string &firstRotor, const std::string &secondRotor, const std::string &thirdRotor, const std::string &cypher)
{
    std::vector<std::string> pairs;
    for (size_t i = 0; i < 10; i++)
    {
        pairs.push_back("AA");
    }

    Plugboard plugboard(pairs);
    Reflector reflector = B;

    double maxIndex = 0;
    std::array<int, 3> maxSettings = {0, 0, 0};

    for (int i = 0; i < 26; i++)
    {
        for (int j = 0; j < 26; j++)
        {
            for (int k = 0; k < 26; k++)
            {
                Rotor left_rotor = rotors.at(firstRotor).Duplicate(0, i);
                Rotor center_rotor = rotors.at(secondRotor).Duplicate(0, j);
                Rotor right_rotor = rotors.at(thirdRotor).Duplicate(0, k);

                Enigma enigma({left_rotor, center_rotor, right_rotor}, reflector, plugboard);

                std::string result = enigma.encrypt(cypher);
                double cypherIndex = index(result);
                if (cypherIndex > maxIndex)
                {
                    maxIndex = cypherIndex;
                    maxSettings = {i, j, k};
                }
            }
        }
    }

    return {maxIndex, maxSettings};
}

struct settings
{
    std::array<int, 3> rotorIndexes;
    std::array<int, 3> rotorPositions;
    double score;
};

bool operator<(settings const &left, settings const &right)
{
    return left.score < right.score;
}

std::array<std::string, 8> rotorNames = {"I", "II", "III", "IV", "V", "VI", "VII", "VIII"};

std::ostream &operator<<(std::ostream &s, const settings &set)
{
    s << rotorNames[set.rotorIndexes[0]] << " " << rotorNames[set.rotorIndexes[1]] << " " << rotorNames[set.rotorIndexes[2]];
    return s << " - " << set.rotorPositions[0] << " " << set.rotorPositions[1] << " " << set.rotorPositions[2] << " : " << set.score;
}

void find_rotor_order(const std::string &cypher, int first_disk)
{
    std::vector<settings> results;
    // for (int i = 4; i < 8; i++)
    // {
    int i = first_disk;
    for (int j = 0; j < 8; j++)
    {
        if (i == j)
            continue;
        for (int k = 0; k < 8; k++)
        {
            if (k == i || k == j)
                continue;
            if (i < 5 && j < 5 && k < 5)
                continue;
            std::cout << "Trying " << rotorNames[i] << ", " << rotorNames[j] << ", " << rotorNames[k] << std::endl;

            auto result = try_all_keys(rotorNames[i], rotorNames[j], rotorNames[k], cypher);
            settings orderedResult;
            orderedResult.rotorIndexes = {i, j, k};
            orderedResult.rotorPositions = result.second;
            orderedResult.score = result.first;
            results.push_back(orderedResult);
        }
    }

    std::sort(results.begin(), results.end());

    for (int i = 0; i < std::min(static_cast<size_t>(10), results.size()); i++)
    {
        std::cout << results[i] << std::endl;
    }
}

int main(int argc, char** argv)
{
    std::stringstream sstream;
    sstream << std::cin.rdbuf();
    std::string cypher = sstream.str();

    // find_rotor_order(cypher, std::atoi(argv[1]));
    // find_rotor_order(cypher, 0);

    std::cout << index(sstream.str()) << std::endl;

    // Rotor left_rotor = rotors.at("I").Duplicate(1, 0);
    // Rotor center_rotor = rotors.at("II").Duplicate(1, 0);
    // Rotor right_rotor = rotors.at("III").Duplicate(1, 0);

    // std::vector<std::string> pairs;
    // for (size_t i = 0; i < 10; i++)
    // {
    //     pairs.push_back("AA");
    // }

    // Plugboard plugboard(pairs);
    // Reflector reflector = B;

    // Enigma enigma({left_rotor, center_rotor, right_rotor}, reflector, plugboard);

    // std::string result = enigma.encrypt("AAAAA");

    // std::cout << result << std::endl;
}
